/*
rndinit.c  Version 0.1.0
Initialize random number generator seed to date
and time in seconds.
Resulting seed is unpredictable.
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

    Free Software Foundation, Inc.
    59 Temple Place - Suite 330
    Boston, MA  02111-1307, USA.
*/

/*
Usage:
        unsigned char *rndSeed;
        rndSeed = (unsigned char *) rndinit();
*/

/*
User subroutine calls:
    void rnd(unsigned char *rndSeed);

System subroutine calls:
    time
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define SEEDLEN 16

unsigned char *rndinit(void)
   {
   int i;
   time_t lTime,*ptrTime;
   unsigned char *rndSeed;
   unsigned char *p,*q;
   int rnd(unsigned char *rndSeed);

   time (&lTime);

   /* allocate memory for seed */
   rndSeed = (unsigned char *) malloc(SEEDLEN+16);
   if (rndSeed == NULL)
      {
      fprintf(stderr,"rndinit: out of memory "
         "allocating rndSeed\n");
      return(NULL);
      } /* out of memory */

   p = rndSeed;
   q = rndSeed + SEEDLEN;
   while (p < q) *p++ = '\0';

   p = rndSeed;
   for (i=0;i<4;i++)
      {
      ptrTime  = &lTime+3;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime;
      } /* for each 4 bytes of the seed */

   p = rndSeed;
   *p |= 1;

   for (i=0;i<4096;i++) rnd(rndSeed);
   return(rndSeed);
   } /* rndinit */
